@@ -316,16 +316,16 @@ class TalosHttpClient(TTransportBase):
 
     def open(self):
         if self.scheme == 'http':
-            self.__http = http_client.HTTPConnection(self.get_addr_by_host(self.host),
-                                                     self.port, timeout=self.__timeout)
+            self.__http = http_client.HTTPConnection(self.get_addr_by_host(self.host), self.port,
+                                                     timeout=self.__timeout)
         elif self.scheme == 'https':
-            self.__http = http_client.HTTPSConnection(self.get_addr_by_host(self.host),
-                                                      self.port, key_file=self.keyfile,
+            self.__http = http_client.HTTPSConnection(self.get_addr_by_host(self.host), self.port,
+                                                      key_file=self.keyfile,
                                                       cert_file=self.certfile,
                                                       timeout=self.__timeout,
                                                       context=self.context)
         if self.using_proxy():
-            self.__http.set_tunnel(self.realhost, self.realport,
+            self.__http.set_tunnel(self.get_addr_by_host(self.realhost), self.realport,
                                    {"Proxy-Authorization": self.proxy_auth})
 
     def close(self):
