@@ -11,6 +11,7 @@ from talos.client.TalosClientConfig import TalosClientConfig
 from talos.client.TalosClientFactory import TalosClientFactory
 from talos.utils import Utils
 from talos.utils.Utils import ReadWriteLock
+from talos.utils.FalconWriter import FalconWriter
 from talos.consumer.MessageProcessorFactory import MessageProcessorFactory
 from talos.consumer.MessageReaderFactory import MessageReaderFactory
 from talos.thrift.consumer.ttypes import ConsumeUnit
@@ -76,6 +77,9 @@ class TalosConsumer(object):
     workerInfoMap = dict
     partitionCheckPoint = dict
 
+    falconWriter = FalconWriter
+    consumerMonitorThread = Timer
+
     def __init__(self, consumerGroup=None, consumerConfig=None,
                  credential=None, topicName=None, messageProcessorFactory=None,
                  clientPrefix=None, talosResourceName=None, workerId=None,
@@ -115,6 +119,7 @@ class TalosConsumer(object):
             self.partitionCheckPoint = partitionCheckPoint
         else:
             self.partitionCheckPoint = dict()
+        self.falconWriter = FalconWriter(falconUrl=consumerConfig.get_falcon_url())
         self.isActive = True
 
         logger.info("The worker: " + self.workerId + " is initializing...")
@@ -130,6 +135,7 @@ class TalosConsumer(object):
         self.init_check_partition_task()
         self.init_check_worker_info_task()
         self.init_renew_task()
+        self.init_consumer_monitor_task()
 
     #
     # Check Partition Task
@@ -175,13 +181,14 @@ class TalosConsumer(object):
             self.set_partition_number(topicPartitionNum)
             # call the re - balance task
             self.reBalanceExecutor = threading.Thread(target=self.re_balance_task)
-            self.reBalanceExecutor.setName("reBalanceTask-" + self.topicName)
+            self.reBalanceExecutor.setName("talos-consumer-reBalanceTask-" + self.topicName)
             self.reBalanceExecutor.start()
 
         self.partitionScheduledExecutor = Timer(interval=self.talosConsumerConfig
                                                 .get_check_partition_interval(),
                                                 function=self.check_partition_task)
-        self.partitionScheduledExecutor.setName("checkPartitionTask-" + self.topicName)
+        self.partitionScheduledExecutor.setName("talos-consumer-checkPartitionTask-"
+                                                + self.topicName)
         self.partitionScheduledExecutor.start()
 
     #
@@ -212,7 +219,8 @@ class TalosConsumer(object):
         self.workerScheduleExecutor = Timer(interval=self.talosConsumerConfig
                                             .get_check_worker_info_interval(),
                                             function=self.check_worker_info_task)
-        self.workerScheduleExecutor.setName("checkWorkerInfoTask-" + self.topicName)
+        self.workerScheduleExecutor.setName("talos-consumer-checkWorkerInfoTask-"
+                                            + self.topicName)
         self.workerScheduleExecutor.start()
 
     #
@@ -282,9 +290,22 @@ class TalosConsumer(object):
         self.renewScheduleExecutor = Timer(interval=self.talosConsumerConfig
                                            .get_renew_check_interval(),
                                            function=self.renew_task)
-        self.renewScheduleExecutor.setName("renewTask-" + self.topicName)
+        self.renewScheduleExecutor.setName("talos-consumer-renewTask-" + self.topicName)
         self.renewScheduleExecutor.start()
 
+    def consumer_monitor_task(self):
+        try:
+            self.push_metric_data()
+        except Exception as throwable:
+            logger.error("push metric data to falcon failed." + str(traceback.format_exc()))
+
+        self.consumerMonitorThread = Timer(interval=self.talosConsumerConfig
+                                           .get_report_metric_interval_millis() / 1000,
+                                           function=self.consumer_monitor_task)
+        self.consumerMonitorThread.setName("talos-consumer-checkPartitionTask-"
+                                           + self.topicName)
+        self.consumerMonitorThread.start()
+
     def check_and_get_topic_info(self, topicTalosResourceName=None):
         topicName = Utils.get_topic_name_by_resource_name(
             str(topicTalosResourceName.topicTalosResourceName.topicTalosResourceName))
@@ -554,23 +575,40 @@ class TalosConsumer(object):
         self.partitionScheduledExecutor = Timer(interval=self.talosConsumerConfig
                                                 .get_check_partition_interval(),
                                                 function=self.check_partition_task)
-        self.partitionScheduledExecutor.setName("checkPartitionTask-" + self.topicName)
+        self.partitionScheduledExecutor.setName("talos-consumer-checkPartitionTask-"
+                                                + self.topicName)
         self.partitionScheduledExecutor.start()
 
     def init_check_worker_info_task(self):
         self.workerScheduleExecutor = Timer(interval=self.talosConsumerConfig
                                             .get_check_worker_info_interval(),
                                             function=self.check_worker_info_task)
-        self.workerScheduleExecutor.setName("checkWorkerInfoTask-" + self.topicName)
+        self.workerScheduleExecutor.setName("talos-consumer-checkWorkerInfoTask-"
+                                            + self.topicName)
         self.workerScheduleExecutor.start()
 
     def init_renew_task(self):
         self.renewScheduleExecutor = Timer(interval=self.talosConsumerConfig
                                            .get_renew_check_interval(),
                                            function=self.renew_task)
-        self.renewScheduleExecutor.setName("renewTask-" + self.topicName)
+        self.renewScheduleExecutor.setName("talos-consumer-renewTask-"
+                                           + self.topicName)
         self.renewScheduleExecutor.start()
 
+    def init_consumer_monitor_task(self):
+        if self.talosConsumerConfig.is_open_client_monitor():
+            self.consumerMonitorThread = Timer(interval=self.talosConsumerConfig
+                                               .get_report_metric_interval_millis() / 1000,
+                                               function=self.consumer_monitor_task)
+            self.consumerMonitorThread.setName("talos-consumer-monitor-" + self.topicName)
+            self.consumerMonitorThread.start()
+
+    def push_metric_data(self):
+        jsonArray = []
+        for key, value in self.partitionFetcherMap.items():
+            jsonArray += value.get_falcon_data()
+        self.falconWriter.push_falcon_data(jsonArray)
+
     #
     # First stop CheckPartition and CheckWorkerInfo Task and then shutdown all fetcher,
     # that avoid thread exit failed caused by rebalance partition when shutDownAllFetcher
@@ -585,5 +623,6 @@ class TalosConsumer(object):
         self.workerScheduleExecutor.cancel()
         self.shut_down_fetcher()
         self.renewScheduleExecutor.cancel()
+        self.consumerMonitorThread.cancel()
         self.isActive = False
         logger.info("Worker: " + self.workerId + " shutdown.")
