@@ -32,6 +32,9 @@ class PartitionMessageQueue:
         self.maxPutMsgBytes = producerConfig.get_max_put_msg_bytes()
         self.globalLock = threading.Condition()
         self.synchronizedLock = threading.Lock()
+        self.should_putLock = threading.Lock()
+        self.get_wait_timeLock = threading.Lock()
+        self.get_all_message_listLock = threading.Lock()
 
     def add_message(self, messageList=None):
         if self.globalLock.acquire():
@@ -91,60 +94,75 @@ class PartitionMessageQueue:
                               " for partition: " + str(self.partitionId))
         return returnList
 
-    @synchronized
     def should_put(self):
-        # when TalosProducer is not active;
-        if not self.producer.is_active():
-            return True
-
-        # when we have enough bytes data or enough number data;
-        if self.curMessageBytes >= self.maxPutMsgBytes or \
-                len(self.userMessageList) >= self.maxPutMsgNumber:
-            return True
-
-        # when there have at least one message and it has exist enough long time;
-        if len(self.userMessageList) > 0:
-            curBufferedTime = Utils.current_time_mills() - self.userMessageList[len(self.userMessageList) - 1].get_timestamp()
-            if curBufferedTime >= self.maxBufferedTime:
+        self.should_putLock.acquire()
+        try:
+            # when TalosProducer is not active;
+            if not self.producer.is_active():
                 return True
 
-        return False
+            # when we have enough bytes data or enough number data;
+            if self.curMessageBytes >= self.maxPutMsgBytes or \
+                    len(self.userMessageList) >= self.maxPutMsgNumber:
+                return True
+
+            # when there have at least one message and it has exist enough long time;
+            if len(self.userMessageList) > 0:
+                curBufferedTime = Utils.current_time_mills() - self.userMessageList[len(self.userMessageList) - 1].get_timestamp()
+                if curBufferedTime >= self.maxBufferedTime:
+                    return True
+
+            return False
+        except Exception as e:
+            self.logger.error("should_put func error: " + traceback.format_exc())
+        finally:
+            self.should_putLock.release()
 
     #
     # Note: wait(0) represents wait infinite until be notified
     # so we wait minimal 1 milli secs when time <= 0
     #
-    @synchronized
     def get_wait_time(self):
-        if len(self.userMessageList) <= 0:
-            return 0
-        time = (self.userMessageList[len(self.userMessageList) - 1].get_timestamp()
-                + self.maxBufferedTime - Utils.current_time_mills()) / 1000
-        if time > 0:
-            return time
-        else:
-            return 1
+        self.get_wait_timeLock.acquire()
+        try:
+            if len(self.userMessageList) <= 0:
+                return 0
+            time = (self.userMessageList[len(self.userMessageList) - 1].get_timestamp()
+                    + self.maxBufferedTime - Utils.current_time_mills()) / 1000
+            if time > 0:
+                return time
+            else:
+                return 1
+        except Exception as e:
+            self.logger.error("get_wait_time func error: " + traceback.format_exc())
+        finally:
+            self.get_wait_timeLock.release()
 
     #
     # clean up MessageQueue, return all message list
     #
-    @synchronized
     def get_all_message_list(self):
-        returnList = []
-        returnMsgBytes = 0
-        returnMsgNumber = 0
+        self.get_all_message_listLock.acquire()
+        try:
+            returnList = []
+            returnMsgBytes = 0
+            returnMsgNumber = 0
 
-        while not len(self.userMessageList) == 0:
-            userMessage = self.userMessageList[len(self.userMessageList) - 1]
-            del self.userMessageList[len(self.userMessageList) - 1]
-            returnList.append(userMessage.get_message())
-            self.curMessageBytes -= userMessage.get_message_size()
-            returnMsgBytes += userMessage.get_message_size()
-            returnMsgNumber += 1
+            while not len(self.userMessageList) == 0:
+                userMessage = self.userMessageList[len(self.userMessageList) - 1]
+                del self.userMessageList[len(self.userMessageList) - 1]
+                returnList.append(userMessage.get_message())
+                self.curMessageBytes -= userMessage.get_message_size()
+                returnMsgBytes += userMessage.get_message_size()
+                returnMsgNumber += 1
 
-        # update total buffered count when poll messageList
-        self.producer.decrease_buffered_count(returnMsgNumber, returnMsgBytes)
-        return returnList
+            # update total buffered count when poll messageList
+            self.producer.decrease_buffered_count(returnMsgNumber, returnMsgBytes)
+            return returnList
+        except Exception as e:
+            self.logger.error("get_all_message_list func error: " + traceback.format_exc())
+        finally:
+            self.get_all_message_listLock.release()
 
     @synchronized
     def get_cur_message_bytes(self):
