@@ -180,6 +180,7 @@ class PartitionSender:
         self.globalLock = globalLock
         self.producer = producer
         self.synchronizedLock = threading.Lock()
+        self.update_stateLock = threading.Lock()
 
         self.curState = TASK_STATE.ACTIVE
         self.topicAndPartition = TopicAndPartition(topicName,
@@ -347,23 +348,28 @@ class PartitionSender:
     def get_cur_state(self):
         return self.curState
 
-    @synchronized
     def update_state(self, targetState=None):
-        self.logger.info("PartitionSender for Partition: " + str(self.partitionId)
-                         + " update status from: " + str(self.curState) + " to: " +
-                         str(targetState))
-        if targetState == TASK_STATE.ACTIVE:
-            self.logger.error("targetState can never be ACTIVE, updateState error for: "
-                              + str(self.partitionId))
+        self.update_stateLock.acquire()
+        try:
+            self.logger.info("PartitionSender for Partition: " + str(self.partitionId)
+                             + " update status from: " + str(self.curState) + " to: " +
+                             str(targetState))
+            if targetState == TASK_STATE.ACTIVE:
+                self.logger.error("targetState can never be ACTIVE, updateState error for: "
+                                  + str(self.partitionId))
+                return
+            if targetState == TASK_STATE.CLOSE:
+                if self.curState == TASK_STATE.ACTIVE:
+                    self.curState = TASK_STATE.CLOSE
+                else:
+                    self.logger.error("targetState is CLOSE, but curState is: " +
+                                      str(self.curState) + " for partition: " +
+                                      str(self.partitionId))
             return
-        if targetState == TASK_STATE.CLOSE:
-            if self.curState == TASK_STATE.ACTIVE:
-                self.curState = TASK_STATE.CLOSE
-            else:
-                self.logger.error("targetState is CLOSE, but curState is: " +
-                                  str(self.curState) + " for partition: " +
-                                  str(self.partitionId))
-        return
+        except Exception as e:
+            self.logger.error("update_state func error: " + traceback.format_exc())
+        finally:
+            self.update_stateLock.release()
 
     def is_flush_finish(self):
         messageQueueSize = self.partitionMessageQueue.get_cur_message_bytes()
