@@ -83,6 +83,11 @@ class TalosProducer(object):
                                                   self.maxBufferedMsgBytes)
         self.clientId = Utils.generate_client_id(producerConfig.get_client_ip(), "producer")
         self.synchronizedLock = threading.Lock()
+        self.add_user_messageLock = threading.Lock()
+        self.do_add_user_messageLock = threading.Lock()
+        self.create_partition_senderLock = threading.Lock()
+        self.release_partition_senderLock = threading.Lock()
+
         self.falconWriter = FalconWriter(falconUrl=self.talosProducerConfig.get_falcon_url())
 
         # getTopicTalosResourceName(producerConfig, credential);
@@ -146,89 +151,99 @@ class TalosProducer(object):
         self.producerMonitorThread.setName("talos-producer-monitor-" + self.topicName)
         self.producerMonitorThread.start()
 
-    @synchronized
     def add_user_message(self, msgList=None, partitionId=None):
-        if not self.is_active():
-            raise ProducerNotActiveException("Producer is not active, current state: "
-                                             + str(self.producerState))
+        self.add_user_messageLock.acquire()
+        try:
+            if not self.is_active():
+                raise ProducerNotActiveException("Producer is not active, current state: "
+                                                 + str(self.producerState))
 
-        # check total buffered message number
-        while self.bufferedCount.is_full():
-            with self.lock:
-                try:
-                    self.logger.info("too many buffered messages, globalLock is active."
-                                     + " message number: " +
-                                     str(self.bufferedCount.get_buffered_msg_number())
-                                     + ", message bytes:  " +
-                                     str(self.bufferedCount.get_buffered_msg_bytes()))
-                    # release global lock but no notify
-                    self.globalLock.acquire()
-                    self.globalLock.wait()
-                    self.globalLock.release()
-                except Exception as e:
-                    if isinstance(e, GalaxyTalosException):
-                        self.logger.error(e.errMsg)
-                    self.logger.error("addUserMessage global lock wait is interrupt."
-                                      + str(traceback.format_exc()))
+            # check total buffered message number
+            while self.bufferedCount.is_full():
+                with self.lock:
+                    try:
+                        self.logger.info("too many buffered messages, globalLock is active."
+                                         + " message number: " +
+                                         str(self.bufferedCount.get_buffered_msg_number())
+                                         + ", message bytes:  " +
+                                         str(self.bufferedCount.get_buffered_msg_bytes()))
+                        # release global lock but no notify
+                        self.globalLock.acquire()
+                        self.globalLock.wait()
+                        self.globalLock.release()
+                    except Exception as e:
+                        if isinstance(e, GalaxyTalosException):
+                            self.logger.error(e.errMsg)
+                        self.logger.error("addUserMessage global lock wait is interrupt."
+                                          + str(traceback.format_exc()))
 
-        if not partitionId:
-            self.logger.debug("add_user_message without assign specific partition id for："
-                              + self.topicName)
-            self.do_add_user_message(msgList)
-        else:
-            userMessageList = []
+            if not partitionId:
+                self.logger.debug("add_user_message without assign specific partition id for："
+                                  + self.topicName)
+                self.do_add_user_message(msgList)
+            else:
+                userMessageList = []
+
+                for message in msgList:
+                    # set timestamp and messageType if not set
+                    Utils.update_message(message, MessageType.BINARY)
+                    # check data validity
+                    Utils.check_message_validity(message)
+                    userMessageList.append(UserMessage(message))
+
+                assert partitionId in self.partitionSenderMap
+                self.partitionSenderMap.get(partitionId).add_message(userMessageList)
+        except Exception as e:
+            self.logger.error("add_user_message func error: " + traceback.format_exc())
+        finally:
+            self.add_user_messageLock.release()
+
+    def do_add_user_message(self, msgList=None):
+        self.do_add_user_messageLock.acquire()
+        try:
+            # user can optionally set 'partitionKey' and 'sequenceNumber' when construct Message
+            partitionBufferMap = dict()
+
+            # check / update curPartitionId
+            if self.should_update_partition():
+                self.curPartitionId = (self.curPartitionId + 1) % self.partitionNumber
+                self.logger.debug("change cur partition id to: " + str(self.curPartitionId))
+                self.lastUpdatePartitionIdTime = Utils.current_time_mills()
+                self.lastAddMsgNumber = 0
+            currentPartitionId = self.curPartitionId
+            self.lastAddMsgNumber += len(msgList)
+
+            partitionBufferMap[currentPartitionId] = []
 
             for message in msgList:
-                # set timestamp and messageType if not set
+                # set timestamp and messageType if not set;
                 Utils.update_message(message, MessageType.BINARY)
                 # check data validity
                 Utils.check_message_validity(message)
-                userMessageList.append(UserMessage(message))
 
-            assert partitionId in self.partitionSenderMap
-            self.partitionSenderMap.get(partitionId).add_message(userMessageList)
-
-    # @synchronized
-    def do_add_user_message(self, msgList=None):
-        # user can optionally set 'partitionKey' and 'sequenceNumber' when construct Message
-        partitionBufferMap = dict()
-
-        # check / update curPartitionId
-        if self.should_update_partition():
-            self.curPartitionId = (self.curPartitionId + 1) % self.partitionNumber
-            self.logger.debug("change cur partition id to: " + str(self.curPartitionId))
-            self.lastUpdatePartitionIdTime = Utils.current_time_mills()
-            self.lastAddMsgNumber = 0
-        currentPartitionId = self.curPartitionId
-        self.lastAddMsgNumber += len(msgList)
-
-        partitionBufferMap[currentPartitionId] = []
-
-        for message in msgList:
-            # set timestamp and messageType if not set;
-            Utils.update_message(message, MessageType.BINARY)
-            # check data validity
-            Utils.check_message_validity(message)
-
-            # check partitionKey setting and validity
-            if not message.partitionKey:
-            # straight forward put to cur partitionId queue
-                userMessage = UserMessage(message)
-                partitionBufferMap.get(currentPartitionId).append(userMessage)
-            else:
-                self.check_message_partition_key_validity(message.getPartitionKey())
-                # construct UserMessage and dispatch to buffer by partitionId
-                partitionId = self.get_partition_id(message.partitionKey)
-                if partitionId not in partitionBufferMap:
-                    partitionBufferMap[partitionId] = []
+                # check partitionKey setting and validity
+                if not message.partitionKey:
+                # straight forward put to cur partitionId queue
                     userMessage = UserMessage(message)
-                    partitionBufferMap.get(partitionId).append(userMessage)
+                    partitionBufferMap.get(currentPartitionId).append(userMessage)
+                else:
+                    self.check_message_partition_key_validity(message.getPartitionKey())
+                    # construct UserMessage and dispatch to buffer by partitionId
+                    partitionId = self.get_partition_id(message.partitionKey)
+                    if partitionId not in partitionBufferMap:
+                        partitionBufferMap[partitionId] = []
+                        userMessage = UserMessage(message)
+                        partitionBufferMap.get(partitionId).append(userMessage)
 
-        # add to partitionSender
-        for key, value in partitionBufferMap.items():
-            partitionId = key
-            assert partitionId in self.partitionSenderMap
-            self.partitionSenderMap.get(partitionId).add_message(value)
+            # add to partitionSender
+            for key, value in partitionBufferMap.items():
+                partitionId = key
+                assert partitionId in self.partitionSenderMap
+                self.partitionSenderMap.get(partitionId).add_message(value)
+        except Exception as e:
+            self.logger.error("do_add_user_message func error: " + traceback.format_exc())
+        finally:
+            self.do_add_user_messageLock.release()
 
     def should_update_partition(self):
         return (Utils.current_time_mills() - self.lastUpdatePartitionIdTime >=
@@ -289,28 +304,38 @@ class TalosProducer(object):
                 self.release_partition_sender(partitionId)
                 partitionId += 1
 
-    @synchronized
     def create_partition_sender(self, partitionId=None):
-        partitionSender = PartitionSender(partitionId=partitionId,
-                                          topicName=self.topicName,
-                                          topicTalosResourceName=
-                                          self.topicTalosResourceName,
-                                          requestId=self.requestId, clientId=self.clientId,
-                                          talosProducerConfig=self.talosProducerConfig,
-                                          messageClient=self.talosClientFactory.
-                                          new_message_client(), userMessageCallback=
-                                          self.userMessageCallback,
-                                          messageCallbackExecutors=
-                                          self.messageCallbackExecutors,
-                                          globalLock=self.globalLock, producer=self)
-        self.partitionSenderMap[partitionId] = partitionSender
-        self.logger.debug("init partition sender for: " + self.topicName
-                          + "for partitionId : " + str(partitionId))
+        self.create_partition_senderLock.acquire()
+        try:
+            partitionSender = PartitionSender(partitionId=partitionId,
+                                              topicName=self.topicName,
+                                              topicTalosResourceName=
+                                              self.topicTalosResourceName,
+                                              requestId=self.requestId, clientId=self.clientId,
+                                              talosProducerConfig=self.talosProducerConfig,
+                                              messageClient=self.talosClientFactory.
+                                              new_message_client(), userMessageCallback=
+                                              self.userMessageCallback,
+                                              messageCallbackExecutors=
+                                              self.messageCallbackExecutors,
+                                              globalLock=self.globalLock, producer=self)
+            self.partitionSenderMap[partitionId] = partitionSender
+            self.logger.debug("init partition sender for: " + self.topicName
+                              + "for partitionId : " + str(partitionId))
+        except Exception as e:
+            self.logger.error("create_partition_sender func error: " + traceback.format_exc())
+        finally:
+            self.create_partition_senderLock.release()
 
-    @synchronized
     def release_partition_sender(self, partitionId=None):
-        self.partitionSenderMap.get(partitionId).close()
-        del self.partitionSenderMap[partitionId]
+        self.release_partition_senderLock.acquire()
+        try:
+            self.partitionSenderMap.get(partitionId).close()
+            del self.partitionSenderMap[partitionId]
+        except Exception as e:
+            self.logger.error("release_partition_sender func error: " + traceback.format_exc())
+        finally:
+            self.release_partition_senderLock.release()
 
     def set_partition_number(self, partitionNum=None):
         self.partitionNumber = partitionNum
